//----------------------------------------------------------------------------------
// File:   Smoke.h
// Author: Ignacio Llamas
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#ifndef _SMOKE_H_
#define _SMOKE_H_

#define STRSAFE_NO_DEPRECATE

#include <tchar.h>
#include "DXUT.h"
#include "NVUTMedia.h"

#ifndef SAFE_ACQUIRE
#define SAFE_ACQUIRE(dst, p)      { if(dst) { SAFE_RELEASE(dst); } if (p) { (p)->AddRef(); } dst = (p); }
#endif

extern D3D10_SHADER_MACRO                 *g_pSmokeShadersMacros;

// Some common globals shared across
extern bool                                g_useFire;
extern int                                 g_Width;
extern int                                 g_Height;
extern float                               g_zNear;
extern float                               g_zFar;
extern bool                                g_renderGlow;
extern float                               g_glowContribution;
extern float                               g_finalIntensityScale; 
extern float                               g_finalAlphaScale;
extern float                               g_smokeColorMultiplier;   
extern float                               g_smokeAlphaMultiplier; 
extern int                                 g_RednessFactor; 
extern float                               g_xyVelocityScale;
extern float                               g_zVelocityScale;
extern D3DXMATRIX                          g_View;
extern D3DXMATRIX                          g_Projection;
extern float                               g_Fovy;

extern ID3D10ShaderResourceView           *g_pSceneDepthSRV;

extern D3DXMATRIX                          g_gridWorld;

extern HRESULT loadTextureFromFile(LPCWSTR file,LPCSTR shaderTextureName, ID3D10Device* pd3dDevice,ID3D10Effect* pEffect);


inline void ComputeRowColsForFlat3DTexture( int depth, int *outCols, int *outRows )
{
    // Compute # of rows and cols for a "flat 3D-texture" configuration
    // (in this configuration all the slices in the volume are spread in a single 2D texture)
    int rows =(int)floorf(sqrtf((float)depth));
    int cols = rows;
    while( rows * cols < depth ) {
        cols++;
    }
    assert( rows*cols >= depth );
    
    *outCols = cols;
    *outRows = rows;
}
#endif // _SMOKE_H_
